package TekitoBasic
{
	import com.kujirahand.utils.KString;
	import com.kujirahand.utils.KTokenizer;
	public class TBLex
	{
		public var tokens:Array; /* of TBToken */
		public var index:int = 0;
		private var parser:TBParser;
		private var reserve:TBReserve;
		public function TBLex(parser:TBParser)
		{
			this.parser = parser;
			this.parser.lex = this;
			this.reserve = new TBReserve(parser);
			tokens = [];
		}
		public function next():TBToken
		{
			return tokens[index++];
		}
		public function splitAll(source:String):void
		{
			var lineno:int = 1;
			source = source.replace(/(\r\n|\r)/g, "\n"); // CRLF -> LF | CR -> LF
			var tok:KTokenizer = new KTokenizer(source);
			tok.getSpace();
			while (tok.source != "") {
				tok.getSpaceCRLF();
				if (tok.source == "") break;
				// label
				var lbl:int = tok.getTokenInt(0);
				if (lbl == 0) throw new Error("Syntax Error : No Label");
				tokens.push(new TBToken(parser.LABEL, lbl, lineno));
				// program
				while (tok.source != "") {
					tok.getSpace();
					var c:String = tok.getChar();
					if (c == "\n") {
						tokens.push(new TBToken(parser.EOL, "\n", lineno));
						lineno++;
						break;
					}
					if (KString.isNumeric(c)) {
						tok.ungetChar(c);
						tokens.push(new TBToken(parser.INT, tok.getTokenInt(0), lineno));
						continue;
					}
					if (KString.isAlpha(c) || c == "_") {
						tok.ungetChar(c);
						var word:String = tok.getTokenVarName();
						word = word.toLowerCase();
						if (reserve.table[word] != undefined) {
							tokens.push(new TBToken(reserve.table[word], word, lineno));
							continue;
						}
						if (TBFunction.table[word] != undefined) {
							tokens.push(new TBToken(parser.FUNCTION, word, lineno));
							continue;
						}
						tokens.push(new TBToken(parser.WORD, word, lineno));
						continue;
					}
					if (c == '"' || c == "'") {
						tok.ungetChar(c);
						var str:String = tok.getTokenString();
						tokens.push(new TBToken(parser.STR, str, lineno));
						continue;
					}
					var c2:String;
					if (c == '>') {
						c2 = tok.getChar();
						if (c2 == '=') {
							tokens.push(new TBToken(parser.GTEQ, ">=", lineno));
						} else if (c == "<") {
							tokens.push(new TBToken(parser.NOTEQ, "<>", lineno));
						}else {
							tok.ungetChar(c2);
							tokens.push(new TBToken(parser.GT, ">", lineno));
						}
						continue;
					}
					if (c == '<') {
						c2 = tok.getChar();
						if (c2 == '=') {
							tokens.push(new TBToken(parser.LTEQ, "<=", lineno));
						} else if (c == ">") {
							tokens.push(new TBToken(parser.NOTEQ, "<>", lineno));
						} else {
							tok.ungetChar(c2);
							tokens.push(new TBToken(parser.LT, "<", lineno));
						}
						continue;
					}
					tokens.push(new TBToken(c.charCodeAt(0), c, lineno));
				}
			}
		}
		public function toString():String
		{
			var s:String = "";
			for each (var i:TBToken in tokens) {
				var sym:int = i.type;
				s += i.value + ":" + parser.yytokname(sym) + "\n";
			}
			return s;
		}
	}
}