%{
// -- head.begin
package TekitoBasic
{
    import flash.display.*;
    import flash.text.*;
    import TekitoBasic.nodes.*;
    public class TBParser
    {
// -- head.end
%}

/* TOKENS */
%token EOL, INT, NUMBER, STR, LABEL, WORD, FUNCTION
%token IF, THEN, ELSE, FOR, TO, NEXT, GOTO
%token OR,AND,MOD,GT,GTEQ,LT,LTEQ,NOTEQ,NOT

%left AND OR
%left GT GTEQ LT LTEQ NOTEQ '='
%left '+' '-'
%left '*' '/' '%' MOD
%left NOT

%%

start	:  lines	{ }
		;
		
lines	: /* empty */{ $$ = topnode; }
    	| lines line { yylrec++; gen.catNode($1, $2); }
    	;
    	
line	: LABEL programs EOL		{ $$ = gen.addLabel($1); gen.catNode($$, $2); }
		| LABEL ':' programs EOL	{ $$ = gen.addLabel($1); gen.catNode($$, $3); }
    	| EOL
    	| error EOL
    	;

programs	: program				{ $$ = $1; }
			| programs ':' program	{ $$ = gen.catNode($1, $3); }
			;
			
program	: FUNCTION args		{ $$ = gen.callFunc($1, $2); }
		| FUNCTION			{ $$ = gen.callFunc($1, null); }
		| if_block
		| goto_block
		| for_block
		| next_block
		| variable '=' expr	{ $$ = gen.let($1, $3); }
    	;

if_block	: IF expr THEN INT			{ $$ = gen.genIf($2, $4, null); }
			| IF expr THEN INT ELSE INT	{ $$ = gen.genIf($2, $4, $6); }
    		;

goto_block	: GOTO INT	{ $$ = gen.genGoto($2); }
			;

for_block	: FOR WORD '=' expr TO expr	{ $$ = gen.genFor($2,$4,$6); }
			;
next_block	: NEXT		{ $$ = gen.genNext(); }
			;
args	: expr			{ $$ = $1; }
		| args ',' expr	{ $$ = gen.catNode($1, $3); }
		;
		
expr	: expr '+' expr { $$ = gen.calcPlus($1, $3); }
    	| expr '-' expr { $$ = gen.calcMinus($1, $3); }
    	| expr '*' expr { $$ = gen.calcMul($1, $3); }
    	| expr '/' expr { $$ = gen.calcDiv($1, $3); }
    	| expr '%' expr { $$ = gen.calcMod($1, $3); }
    	| expr MOD expr { $$ = gen.calcMod($1, $3); }
    	| expr GT expr	{ $$ = gen.calcGt($1, $3); }
    	| expr GTEQ expr{ $$ = gen.calcGtEq($1, $3); }
    	| expr LT expr 	{ $$ = gen.calcLt($1, $3); }
    	| expr LTEQ expr	{ $$ = gen.calcLtEq($1, $3); }
    	| expr NOTEQ expr	{ $$ = gen.calcNotEq($1, $3); }
    	| expr '=' expr	{ $$ = gen.calcEq($1, $3); }
    	| expr AND expr	{ $$ = gen.calcAnd($1, $3); }
    	| expr OR expr	{ $$ = gen.calcOr($1, $3); }
    	| '(' expr ')'	{ $$ = $2; }
    	| NUMBER	{ $$ = gen.pushNumber($1); }
    	| INT		{ $$ = gen.pushInt($1); }
    	| STR		{ $$ = gen.pushStr($1); }
    	| variable	{ $$ = gen.pushVariable($1); }
		;

variable: WORD					{ $$ = gen.getVariable($1); }
		| variable '[' expr ']' { $$ = gen.getArray($1, $3); }
		;
%%

// -- footer.begin
    	public function TBParser()
    	{
    	}
    	public function compile():Boolean
    	{
    		// option
	        yydebug 			= true;
	        yyDumpParseTree 	= true;
	        // parser
	        topnode = new TBNode();
	        topnode.token = new TBToken(0, "[topnode]", 0);
	        gen = new TBParserGen(this);
	        yyparse();
	        return result;
    	}
    	public function run():Boolean
    	{
    		return true;
    	}
		// lex
		public var lex:TBLex;
		public var topnode:TBNode;
		private var cur_token:TekitoBasic.TBToken;
		private var gen: TBParserGen;
		private var result:Boolean = true;
		public var error_message:String = "";
		private function yylex():int {
			yylval = null;
			cur_token = lex.next();
			if (cur_token == null) return 0;
			yylval = cur_token;
			return cur_token.type;
		}
    	//
		public var labels:Object = {};
		//
		private function yyerror(msg:String):void {
			result = false;
	  		var s:String = "[ERROR](" + cur_token.lineno + "): " + msg + "\n";
	  		trace(s);
			error_message += s;
		}
// -- footer.end

