package TekitoBasic
{	
	import TekitoBasic.nodes.*;
	
	public class TBParserGen
	{
		private var parser:TBParser;
		public function TBParserGen(parser:TBParser)
		{
			this.parser = parser;
		}
		public function addLabel(t:Object):TBNode
		{
			var n:TBNode = new TBNode();
			n.token = t as TBToken;
			var label:String = String(t.value);
			n.token.value = "LABEL:" + label;
			parser.labels[label] = n;
			return n;
		}
		public function catNode(cur:Object, add_n:Object):TBNode
		{
			var n:TBNode = cur as TBNode;
			while (n.next != null) {
				n = n.next;
			}
			n.next = add_n as TBNode;
			return cur as TBNode;
		}
		public function catNodeArray(nodes:Array):TBNode
		{
			if (nodes.length == 0) return null;
			var top:TBNode = nodes[0];
			var last:TBNode = top;
			for (var i:int = 1; i < nodes.length; i++) {
				catNode(last, nodes[i]);
				last = nodes[i];
			}
			return top;
		}
		public function callFunc(tok:TBToken, args:TBNode):TBNode
		{
			var n:TBNode = new TBNodeCallFunction();
			n.token = tok;
			n.items[0] = args;
			var link:Object = TBFunction.table[ tok.value ];
			TBNodeCallFunction(n).link = link;
			return n;
		}
		public function pushInt(tok:TBToken):TBNode
		{
			var n:TBNode = new TBNodeInt();
			n.token = tok;
			return n;
		}
		public function pushStr(tok:TBToken):TBNode
		{
			var n:TBNode = new TBNodeInt();
			n.token = tok;
			return n;
		}
		public function pushNumber(tok:TBToken):TBNode
		{
			var n:TBNode = new TBNodeNumber();
			n.token = tok;
			return n;
		}
		public function genCalc(n1:TBNode, n2:TBNode, flag:String, klass:Class):TBNode
		{
			var n:TBNode = new klass();
			n.token.value = flag;
			n.token.lineno = n1.token.lineno;
			n.items[0] = n1;
			n.items[1] = n2;
			return n;
		}
		public function calcPlus(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "+", TBNodeCalcPlus);
		}
		public function calcMinus(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "-", TBNodeCalcMinus);
		}
		public function calcMul(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "*", TBNodeCalcMul);
		}
		public function calcDiv(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "/", TBNodeCalcDiv);
		}
		public function calcMod(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "mod", TBNodeCalcMod);
		}
		public function calcGt(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, ">", TBNodeCalcGt);
		}
		public function calcGtEq(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, ">=", TBNodeCalcGtEq);
		}
		public function calcLt(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "<", TBNodeCalcLt);
		}
		public function calcLtEq(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "<=", TBNodeCalcLtEq);
		}
		public function calcNotEq(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "<>", TBNodeCalcNotEq);
		}
		public function calcAnd(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "and", TBNodeCalcAnd);
		}
		public function calcOr(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "or", TBNodeCalcOr);
		}
		public function calcNot(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "not", TBNodeCalcNot);
		}
		public function calcEq(n1:TBNode, n2:TBNode):TBNode
		{
			return genCalc(n1, n2, "=", TBNodeCalcEq);
		}
		public function getArray(v:TBNode, expr:TBNode):TBNode
		{
			var r:TBNode = new TBNodeGetArray();
			r.token.lineno = v.token.lineno;
			r.token.value = "getArray";
			r.items[0] = v;
			r.items[1] = expr;
			return r;
		}
		public function getVariable(n:TBToken):TBNode
		{
			var r:TBNode = new TBNodeGetVariable();
			r.token.value = "getVar";
			r.token = n;
			return r;
		}
		public function let(v:TBNode, expr:TBNode):TBNode
		{
			var r:TBNode = new TBNodeLet();
			r.token.value = "let";
			r.token.lineno = v.token.lineno;
			r.items[0] = v;
			r.items[1] = expr;
			return r;
		}
		public function pushVariable(v:TBNode):TBNode
		{
			var r:TBNode = new TBNodePushVariable();
			r.token = v.token;
			r.items[0] = v;
			return r;
		}
		public function genIf(expr:TBNode, true_n:TBToken, false_n:TBToken):TBNode
		{
			var r:TBNode = new TBNodeIf();
			r.token.lineno = expr.token.lineno;
			r.token.value = "if";
			r.items[0] = expr;
			r.items[1] = true_n;
			r.items[2] = false_n;
			return r;
		}
		public function genGoto(lbl:TBToken):TBNode
		{
			var r:TBNode = new TBNodeGoto();
			r.token = lbl;
			return r;
		}
		private static var jump_point_num:int = 1;
		public function addJumpPoint():TBNode
		{
			var n:TBNode = new TBNode();
			var name:String = "JUMP_POINT_" + String(jump_point_num++);
			n.token.value = name;
			parser.labels[name] = n;
			return n;
		}
		public function genFor(v:TBToken, from_n:TBNode, to_n:TBNode):TBNode
		{
			var n:TBNode = new TBNodeFor();
			n.items = [v, from_n, to_n];
			return n;
		}
		public function genNext():TBNode
		{
			var r:TBNode = new TBNodeNext();
			return r;
		}
	}
}