package com.kujirahand.utils
{
	public class KString
	{
        /**
         * 文字列 str の a を b に置換する
         * @param str 置換対象文字列
         * @param a   検索文字列
         * @param b   置換文字列
         * @return    置換結果を返す
         */
        public static function replace(str:String,
                                  a:String, b:String):String {
            var str_ary:Array = str.split(a);
            return str_ary.join(b);
        }
        /**
         * 文字列 str の左から n 文字を取り出す
         * @param str 対象文字列
         * @param n   何文字か
         * @return strの左からn文字
         */
        public static function left(str:String, n:int):String {
            return str.substr(0, n);
        }
        /**
         * 文字列 str の右から n 文字を取り出す
         * @param str   対象文字列
         * @param n     n文字
         * @return      strの右からn文字
         */
        public static function right(str:String, n:int):String {
            return str.substr(str.length - n);
        }
        /**
         * 置換対象文字列オブジェクト str_obj.str から
         * 区切り記号 splitter までを切り取って返す
         * @param str_obj   置換対象を文字列strを含むオブジェクト
         * @param splitter  区切り文字列
         * @return str_obj.str から splitterの直前までの文字列
         * @example 文字列"aaa.bbb=ccc"を区切ります
         * var obj:Object = {str:"abc.def=ghi"};
         * trace(KString.getToken(obj, "."));
         * trace(KString.getToken(obj, "="));
         * trace(obj.str);
         */
        public static function getToken(str_obj:Object,
                                  splitter:String):String {
            // splitter までを区切る
            var token_ary:Array = str_obj.str.split(splitter, 1);
            var res:String = token_ary[0];
            // 切り取った文字列を削除する
            str_obj.str = str_obj.str.substr(res.length + splitter.length);
            // 切り取った分を戻り値とする
            return res;
        }
        /**
         * ひらがなをカタカナに変換する
         * @param str   置換対象文字列
         * @return      置換後の文字列
         */
        public static function convertHiragaToKatakana(
                                          str:String):String {
            var code_from:int    = "ぁ".charCodeAt(0);
            var code_to:int      = "ん".charCodeAt(0);
            var code_replace:int = "ァ".charCodeAt(0);
            var result:String = "";
            for (var i:int = 0; i < str.length; i++) {
                var c:int = str.charCodeAt(i);
                if (code_from <= c && c <= code_to) {
                    c = code_replace + (c - code_from);
                }
                result += String.fromCharCode(c);
            }
            return result;
        }
        /**
         * カタカナをひらがなに変換する
         * @param   str   置換対象文字列
         * @return        置換後の文字列
         */
        public static function convertKatakanaToHiragana(str:String):String {
            var code_from:int    = "ァ".charCodeAt(0);
            var code_to:int      = "ン".charCodeAt(0);
            var code_replace:int = "ぁ".charCodeAt(0);
            var result:String = "";
            for (var i:int = 0; i < str.length; i++) {
                var c:int = str.charCodeAt(i);
                if (code_from <= c && c <= code_to) {
                    c = code_replace + (c - code_from);
                }
                result += String.fromCharCode(c);
            }
            return result;
        }
        /**
        * タブ文字を展開する
        * @param	source	置換対象文字列
        * @param	tab		タブ幅(省略すると4)
        */
        public static function expandTab(source:String, tab:int = 4):String
        {
        	var res:String = "";
        	var index:int = 0;
        	for (var i:int = 0; i < source.length; i++) {
        		var ch:String = source.charAt(i);
        		if (ch == "\r" || ch == "\n") {
        			index = 0;
        			res += ch;
        			continue;
        		}
        		if (ch == "\t") {
        			var fill:int = index % tab;
        			if (fill == 0) { fill = tab; }
        			res += repeatStr(" ", fill);
        			continue;
        		}
        		index += getCharWidth(ch);
        		res += ch;
        	}
        	return res;
        }
        /**
        * 文字列が半角でいくつ分かを求める
        */
        public static function getCharWidth(source:String):int
        {
        	var count:int = 0;
        	for (var i:int = 0; i < source.length; i++) {
        		var c:Number = source.charCodeAt(i);
        		if (c >= 256) { count += 2; } else { count += 1; }
        	}
        	return count;
        }
        /**
        * 文字列を指定回数繰り返して返す
        * @param	str		文字列
        * @param	count	回数
        */
        public static function repeatStr(str:String, count:int):String
        {
        	var res:String = "";
        	for (var i:int = 0; i < count; i++) {
        		res += str;
        	}
        	return res;
        }
		/**
		 * trim string space
		 * @param	s	string
		 * @return		triming string
		 */
		public static function trim(s:String):String
		{
			return ltrim(rtrim(s));
		}
		public static function ltrim(s:String):String
		{
			var p:int = 0;
			for (var i:int = 0; i < s.length; i++) {
				var c:String = s.charAt(i);
				if (c == " " || c == "\t") continue;
				p = i;
				break;
			}
			return s.substr(p);
		}
		public static function rtrim(s:String):String
		{
			var p:int = 0;
			for (var i:int = 0; i < s.length; i++) {
				var c:String = s.charAt(s.length - i - 1);
				if (c == " " || c == "\t") continue;
				p = i;
				break;
			}
			return s.substr(0, s.length - p);
		}
		/**
		 * get file extension (includes '.')
		 * @param	path
		 * @return	extension
		 */
		public static function getFileExt(path:String):String
		{
			if (path == null || path == "") return "";
			for (var i:int = (path.length - 1); i >= 0; i--) {
				var c:String = path.substr(i, 1);
				if (c == ".") {
					return path.substr(i);
				}
			}
			return "";
		}
		/**
		 * 行末のパス記号を切り落とす
		 * @param	path
		 * @param	flag	path flag, default = '/'
		 * @return	path
		 */
		public static function chopPath(path:String, flag:String = "/"):String
		{
			if (path.substr(path.length - 1, 1) == flag) {
				path = path.substr(0, path.length - 1);
			}
			return path;
		}
		/**
		 * パスのディレクトリ部分があれば返す
		 */
		public static function getBaseDir(path:String, flag:String = "/"):String
		{
			path = KString.chopPath(path, flag);
			var path_ary:Array = path.split(flag);
			path_ary.splice(0, path_ary.length - 1);
			return path_ary.join(flag);
		}
		/**
		 * is numeric
		 * @param ch	string
		 * @return true or false
		 */
		 public static function isNumeric(ch:String):Boolean
		 {
		 	return ('0' <= ch && ch <= '9');
		 }
		/**
		 * is alphabet
		 * @param ch	string
		 * @return true or false
		 */
		 public static function isAlpha(ch:String):Boolean
		 {
		 	return ('A' <= ch && ch <= 'Z') || ('a' <= ch && ch <= 'z');
		 }
	}
}