package com.kujirahand.utils
{
	/**
	 * 文字列をトークンに区切るためのクラス
	 */
	public class KTokenizer
	{
		private var _source:String;
		
		public function KTokenizer(source:String)
		{
			this._source = source;
		}
		
		public function get source():String
		{
			return _source;
		}
		/**
		 * 特定の区切り記号までをトークンとして切り出す。切り出した部分はソースから削除される。
		 */
		public function getToSplitter(splitter:String):String
		{
			var ary:Array = _source.split(splitter, 1);
			var ret:String = ary[0];
			_source = _source.substr(ret.length + splitter.length);
			return ret;
		}
		/**
		 * 任意のパターン(正規表現)にマッチする部分までを切り出す。切り出した部分はソースから削除される。
		 */
		public function getPatternStr(pattern:String, flag:String=""): String
		{
			if (pattern.substr(0,1) != "^") {
				pattern = "^" + pattern;
			}
			var re:RegExp = new RegExp(pattern, flag);
			return getPattern(re);
		}
		/**
		 * 任意のパターン(正規表現)にマッチする部分までを切り出す。切り出した部分はソースから削除される。
		 */
		public function getPattern(pattern:RegExp):String
		{
			var result:Object = pattern.exec(_source);
			if (result == null) return ""; // failed
			var ret:String = result[0];
			_source = _source.substr(result.index + ret.length);
			return ret;
		}
		/**
		 * 任意のパターンにマッチするかテストする
		 */
		public function test(pattern:RegExp):Boolean
		{
			return pattern.test(_source);
		}
		/**
		 * 任意の文字列にマッチするかテストする
		 */
		public function testStr(str:String):Boolean
		{
			return (_source.substr(0, str.length) == str);
		}
		/**
		 * 一文字だけ切り出し、ソースから削除する
		 */
		public function getChar():String
		{
			var c:String = _source.charAt(0);
			_source = _source.substr(1);
			return c;
		}
		/**
		 * 一文字だけ切り出し、ソースから削除する
		 */
		public function getCharCode():Number
		{
			var c:Number = _source.charCodeAt(0);
			_source = _source.substr(1);
			return c;
		}
		public function ungetChar(ch:String):void
		{
			_source = ch + _source;
		}
		//--- 各種トークンを取り出す
		private static var pat_int:RegExp 		= undefined;
		private static var pat_xmlname:RegExp 	= /^[\w\_][\w\d\_\-\:]*/;
		private static var pat_varname:RegExp   = /^[\w\_][\w\d\_]*/;
		/**
		 * 整数を切り出す
		 */
		public function getTokenInt(default_no:int):int
		{
			if (!pat_int) { pat_int = /\d+/; }
			var str:String = getPattern(pat_int);
			if (str == null) {
				return default_no;
			}
			else {
				return int(str);
			}
		}
		/**
		 * XMLの名前を切り出す
		 */
		public function getTokenXmlName():String
		{
			return getPattern(pat_xmlname);
		}
		public function testXmlName():Boolean
		{
			return test(pat_xmlname);
		}
		/**
		 * 変数の名前を切り出す
		 */
		public function getTokenVarName():String
		{
			return getPattern(pat_varname);
		}
		public function testVarName():Boolean
		{
			return test(pat_varname);
		}
		/**
		 * 文字列を切り出す
		 */
		public function getTokenString():String
		{
			var result:String = "";
			var flag:String = this.getChar();
			while (_source != "") {
				var ch:String = this.getChar();
				if (ch == "\\") {
					result += ch + this.getChar();
					continue;
				}
				if (ch == flag) {
					break;
				}
				result += ch;
			}
			return result;
		}
		/**
		 * 先頭にあるスペースを切り取る
		 */
		public function getSpace():String
		{
			var space:String = "";
			for(;;) {
				var ch:String = _source.charAt(0);
				if (ch == " "||ch == "\t") {
					_source = _source.substr(1);
					space += ch;
					continue;
				}
				break;
			}
			return space;
		}
		public function getSpaceCRLF():String
		{
			var space:String = "";
			for(;;) {
				var ch:String = _source.charAt(0);
				if (ch == " "||ch == "\t"||ch == "\r"||ch == "\n") {
					_source = _source.substr(1);
					space += ch;
					continue;
				}
				break;
			}
			return space;
		}
	}
}


